--------------------------------------------------------------------------
--
--  flapcpld.vhd
--  ISA bus inteface logic for FreeIO.org FLAPJACK Programmable IO Board
--  Version 0.11
--  15 April 2000
--  Copyright 2000 Diehl Martin
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--
--------------------------------------------------------------------------


--------------------------------------------------------------------------
--                                                      
--  Library Declaration:
--
--  The library declaration is used to declare the source libraries
--  from which certain necessary primitives are imported.
--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------
--                                                      
--  Entity Declaration:
--
--  The entity declaration is used to describe the inputs and outputs
--  of a design entity.
--
--------------------------------------------------------------------------

entity flapcpld is port
	(
	resin, ior_n, iow_n, aen 
	: in std_logic;
	
	addr  : in    std_logic_vector (9  downto 1);
	sw    : in    std_logic_vector (2  downto 0);
	data  : inout std_logic_vector (15 downto 0);
	xdata : inout std_logic_vector (15 downto 0);
	xaddr : out   std_logic_vector (15 downto 0);
	
	xce_n, xwe_n, xrd_n, io16bit
	: out std_logic 
	);
end flapcpld;
	
--------------------------------------------------------------------------
--                                                      
--  Architecture Declaration:
--
--  The architecture declaration is associated with an entity declaration.
--  An architecture describes the entity's function.
--
--------------------------------------------------------------------------

architecture archflapcpld of flapcpld is
	signal read_n:   std_logic;
	signal write_n:  std_logic;
	signal iosel:    std_logic;
	signal ireg0:    std_logic;
	signal ireg1:    std_logic;
	signal ireg2:    std_logic;
	signal addrreg : std_logic_vector (15 downto 0);
begin

--------------------------------------------------------------------------
----                                                      
----  Identities
----
----  These combinatorial statements are declared to be always true.
----
--------------------------------------------------------------------------

	xaddr       <= addrreg;
	xwe_n       <= not(ireg0 and (not iow_n) and (not aen));
	write_n     <= not(ireg0 and (not iow_n) and (not aen));
	xrd_n       <= not(ireg0 and (not ior_n) and (not aen)); 
	read_n      <= not(ireg0 and (not ior_n) and (not aen));
	xce_n       <= not(ireg0 and (not (iow_n and ior_n)) and (not aen));
	io16bit     <= (iosel and (not (iow_n and ior_n)) and (not aen));

--------------------------------------------------------------------------
--                                                      
--  Bus Direction Control
--
--------------------------------------------------------------------------

	xdata <= data when (write_n = '0') else "ZZZZZZZZZZZZZZZZ";
	data <= xdata when (read_n = '0') else "ZZZZZZZZZZZZZZZZ";
	


--------------------------------------------------------------------------
--                                                      
--  Process Declaration: addr_reg
--
--  This process loads the Address Register "addrreg" from the
--  processor data bus. Note the initialization of all bits to 0. 
--
--------------------------------------------------------------------------

addr_reg : process (resin, ireg1, data) 
	begin
	if (resin = '1') then
		addrreg <= "0000000000000000";
	elsif rising_edge(ireg1) then
		addrreg <= data;
	end if;
end process addr_reg;	

--------------------------------------------------------------------------
--                                                      
--  Process Declaration: io_select
--
--  This process decodes address lines 1 through 9 to determine when
--  the card is accessed by the processor.    
--
--------------------------------------------------------------------------

io_select : process (resin, addr, sw) 
	begin
	if (resin = '1') then
		iosel <= '0';
	elsif addr(9 downto 7) = "100" and addr(3) = '0' then
		if addr(6 downto 4) = sw(2 downto 0) then
			iosel <= '1';
		else
			iosel <= '0';
		end if;
	else
		iosel <= '0';
	end if;
end process io_select;	

--------------------------------------------------------------------------
--                                                      
--  Process Declaration: controls
--
--  This process decodes address lines 1 through 3 to determine when
--  the each port is accessed by the processor.    
--
--------------------------------------------------------------------------

controls : process (resin, addr, iosel, aen, iow_n) 
	begin
	if (resin = '1') then
		ireg0    <= '0';
		ireg1    <= '1';
		ireg2    <= '1';
	elsif  iosel = '1' and aen = '0' then
		if    addr(3 downto 1) = "000" then
			ireg0    <= '1';
			ireg1    <= '1';
			ireg2    <= '1';
		elsif addr(3 downto 1) = "001" then
			ireg0    <= '0';
			ireg1    <= iow_n;
			ireg2    <= '1';
		elsif addr(3 downto 1) = "010" then
			ireg0    <= '0';
			ireg1    <= '1';
			ireg2    <= iow_n;
		else
			ireg0    <= '0';
			ireg1    <= '1';
			ireg2    <= '1';
		end if;
	else
		ireg0     <= '0';
		ireg1     <= '1';
		ireg2     <= '1';
	
	end if;
end process controls;	
end archflapcpld;